"""
merge_any
=========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class merge_any(Operator):
    """Merges a list of objects having the same data types. Once the data
    type is found, the merge operation is forwarded to the correct
    merge Operator.

    Parameters
    ----------
    any1 : Any
        Either a vector of objects (sharing the same
        data types) or objects from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.
    any2 : Any
        Either a vector of objects (sharing the same
        data types) or objects from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.merge_any()

    >>> # Make input connections
    >>> my_any1 = dpf.Any()
    >>> op.inputs.any1.connect(my_any1)
    >>> my_any2 = dpf.Any()
    >>> op.inputs.any2.connect(my_any2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.merge_any(
    ...     any1=my_any1,
    ...     any2=my_any2,
    ... )

    >>> # Get output data
    >>> result_any = op.outputs.any()
    """

    def __init__(self, any1=None, any2=None, config=None, server=None):
        super().__init__(name="merge::any", config=config, server=server)
        self._inputs = InputsMergeAny(self)
        self._outputs = OutputsMergeAny(self)
        if any1 is not None:
            self.inputs.any1.connect(any1)
        if any2 is not None:
            self.inputs.any2.connect(any2)

    @staticmethod
    def _spec():
        description = """Merges a list of objects having the same data types. Once the data
            type is found, the merge operation is forwarded to the
            correct merge Operator."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="any",
                    type_names=["any"],
                    optional=False,
                    document="""Either a vector of objects (sharing the same
        data types) or objects from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.""",
                ),
                1: PinSpecification(
                    name="any",
                    type_names=["any"],
                    optional=False,
                    document="""Either a vector of objects (sharing the same
        data types) or objects from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="any",
                    type_names=["any"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="merge::any", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMergeAny
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMergeAny
        """
        return super().outputs


class InputsMergeAny(_Inputs):
    """Intermediate class used to connect user inputs to
    merge_any operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_any()
    >>> my_any1 = dpf.Any()
    >>> op.inputs.any1.connect(my_any1)
    >>> my_any2 = dpf.Any()
    >>> op.inputs.any2.connect(my_any2)
    """

    def __init__(self, op: Operator):
        super().__init__(merge_any._spec().inputs, op)
        self._any1 = Input(merge_any._spec().input_pin(0), 0, op, 0)
        self._inputs.append(self._any1)
        self._any2 = Input(merge_any._spec().input_pin(1), 1, op, 1)
        self._inputs.append(self._any2)

    @property
    def any1(self):
        """Allows to connect any1 input to the operator.

        Either a vector of objects (sharing the same
        data types) or objects from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.

        Parameters
        ----------
        my_any1 : Any

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_any()
        >>> op.inputs.any1.connect(my_any1)
        >>> # or
        >>> op.inputs.any1(my_any1)
        """
        return self._any1

    @property
    def any2(self):
        """Allows to connect any2 input to the operator.

        Either a vector of objects (sharing the same
        data types) or objects from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.

        Parameters
        ----------
        my_any2 : Any

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_any()
        >>> op.inputs.any2.connect(my_any2)
        >>> # or
        >>> op.inputs.any2(my_any2)
        """
        return self._any2


class OutputsMergeAny(_Outputs):
    """Intermediate class used to get outputs from
    merge_any operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_any()
    >>> # Connect inputs : op.inputs. ...
    >>> result_any = op.outputs.any()
    """

    def __init__(self, op: Operator):
        super().__init__(merge_any._spec().outputs, op)
        self._any = Output(merge_any._spec().output_pin(0), 0, op)
        self._outputs.append(self._any)

    @property
    def any(self):
        """Allows to get any output of the operator

        Returns
        ----------
        my_any : Any

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_any()
        >>> # Connect inputs : op.inputs. ...
        >>> result_any = op.outputs.any()
        """  # noqa: E501
        return self._any



"""
intersect
=========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class intersect(Operator):
    """Intersect 2 scopings and return the intersection and the difference
    between the intersection and the first scoping.

    Parameters
    ----------
    scopingA : Scoping
    scopingB : Scoping


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.intersect()

    >>> # Make input connections
    >>> my_scopingA = dpf.Scoping()
    >>> op.inputs.scopingA.connect(my_scopingA)
    >>> my_scopingB = dpf.Scoping()
    >>> op.inputs.scopingB.connect(my_scopingB)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.intersect(
    ...     scopingA=my_scopingA,
    ...     scopingB=my_scopingB,
    ... )

    >>> # Get output data
    >>> result_intersection = op.outputs.intersection()
    >>> result_scopingA_min_intersection = op.outputs.scopingA_min_intersection()
    """

    def __init__(self, scopingA=None, scopingB=None, config=None, server=None):
        super().__init__(name="scoping::intersect", config=config, server=server)
        self._inputs = InputsIntersect(self)
        self._outputs = OutputsIntersect(self)
        if scopingA is not None:
            self.inputs.scopingA.connect(scopingA)
        if scopingB is not None:
            self.inputs.scopingB.connect(scopingB)

    @staticmethod
    def _spec():
        description = """Intersect 2 scopings and return the intersection and the difference
            between the intersection and the first scoping."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="scopingA",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="scopingB",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="intersection",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="scopingA_min_intersection",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="scoping::intersect", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIntersect
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsIntersect
        """
        return super().outputs


class InputsIntersect(_Inputs):
    """Intermediate class used to connect user inputs to
    intersect operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.intersect()
    >>> my_scopingA = dpf.Scoping()
    >>> op.inputs.scopingA.connect(my_scopingA)
    >>> my_scopingB = dpf.Scoping()
    >>> op.inputs.scopingB.connect(my_scopingB)
    """

    def __init__(self, op: Operator):
        super().__init__(intersect._spec().inputs, op)
        self._scopingA = Input(intersect._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._scopingA)
        self._scopingB = Input(intersect._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scopingB)

    @property
    def scopingA(self):
        """Allows to connect scopingA input to the operator.

        Parameters
        ----------
        my_scopingA : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> op.inputs.scopingA.connect(my_scopingA)
        >>> # or
        >>> op.inputs.scopingA(my_scopingA)
        """
        return self._scopingA

    @property
    def scopingB(self):
        """Allows to connect scopingB input to the operator.

        Parameters
        ----------
        my_scopingB : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> op.inputs.scopingB.connect(my_scopingB)
        >>> # or
        >>> op.inputs.scopingB(my_scopingB)
        """
        return self._scopingB


class OutputsIntersect(_Outputs):
    """Intermediate class used to get outputs from
    intersect operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.intersect()
    >>> # Connect inputs : op.inputs. ...
    >>> result_intersection = op.outputs.intersection()
    >>> result_scopingA_min_intersection = op.outputs.scopingA_min_intersection()
    """

    def __init__(self, op: Operator):
        super().__init__(intersect._spec().outputs, op)
        self._intersection = Output(intersect._spec().output_pin(0), 0, op)
        self._outputs.append(self._intersection)
        self._scopingA_min_intersection = Output(intersect._spec().output_pin(1), 1, op)
        self._outputs.append(self._scopingA_min_intersection)

    @property
    def intersection(self):
        """Allows to get intersection output of the operator

        Returns
        ----------
        my_intersection : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> # Connect inputs : op.inputs. ...
        >>> result_intersection = op.outputs.intersection()
        """  # noqa: E501
        return self._intersection

    @property
    def scopingA_min_intersection(self):
        """Allows to get scopingA_min_intersection output of the operator

        Returns
        ----------
        my_scopingA_min_intersection : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> # Connect inputs : op.inputs. ...
        >>> result_scopingA_min_intersection = op.outputs.scopingA_min_intersection()
        """  # noqa: E501
        return self._scopingA_min_intersection



"""
workflow_energy_per_component
=============================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class workflow_energy_per_component(Operator):
    """Calculates the cumulated energy per component (Named Selection). For
    cyclic and multistage models, the expansion will be automatically
    done.

    Parameters
    ----------
    time_scoping : Scoping, optional
    mesh_scoping : Scoping, optional
        Master scoping. all named selections will be
        intersected with this scoping.
    energy_type : int, optional
        Type of energy to be processed: (0: strain +
        kinetic energy (default), 1: strain
        energy, 2: kinetic energy)
    data_sources : DataSources
    named_selection1 : str, optional
        Named selections. intersection of all  named
        selections with the master scoping
        will be done.
    named_selection2 : str, optional
        Named selections. intersection of all  named
        selections with the master scoping
        will be done.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.workflow_energy_per_component()

    >>> # Make input connections
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_energy_type = int()
    >>> op.inputs.energy_type.connect(my_energy_type)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_named_selection1 = str()
    >>> op.inputs.named_selection1.connect(my_named_selection1)
    >>> my_named_selection2 = str()
    >>> op.inputs.named_selection2.connect(my_named_selection2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.workflow_energy_per_component(
    ...     time_scoping=my_time_scoping,
    ...     mesh_scoping=my_mesh_scoping,
    ...     energy_type=my_energy_type,
    ...     data_sources=my_data_sources,
    ...     named_selection1=my_named_selection1,
    ...     named_selection2=my_named_selection2,
    ... )

    >>> # Get output data
    >>> result_component_energy = op.outputs.component_energy()
    >>> result_component_energy_percentage = op.outputs.component_energy_percentage()
    """

    def __init__(
        self,
        time_scoping=None,
        mesh_scoping=None,
        energy_type=None,
        data_sources=None,
        named_selection1=None,
        named_selection2=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="workflow_energy_per_component", config=config, server=server
        )
        self._inputs = InputsWorkflowEnergyPerComponent(self)
        self._outputs = OutputsWorkflowEnergyPerComponent(self)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if energy_type is not None:
            self.inputs.energy_type.connect(energy_type)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if named_selection1 is not None:
            self.inputs.named_selection1.connect(named_selection1)
        if named_selection2 is not None:
            self.inputs.named_selection2.connect(named_selection2)

    @staticmethod
    def _spec():
        description = """Calculates the cumulated energy per component (Named Selection). For
            cyclic and multistage models, the expansion will be
            automatically done."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""Master scoping. all named selections will be
        intersected with this scoping.""",
                ),
                2: PinSpecification(
                    name="energy_type",
                    type_names=["int32"],
                    optional=True,
                    document="""Type of energy to be processed: (0: strain +
        kinetic energy (default), 1: strain
        energy, 2: kinetic energy)""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""""",
                ),
                5: PinSpecification(
                    name="named_selection",
                    type_names=["string"],
                    optional=True,
                    document="""Named selections. intersection of all  named
        selections with the master scoping
        will be done.""",
                ),
                6: PinSpecification(
                    name="named_selection",
                    type_names=["string"],
                    optional=True,
                    document="""Named selections. intersection of all  named
        selections with the master scoping
        will be done.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="component_energy",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="component_energy_percentage",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="workflow_energy_per_component", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsWorkflowEnergyPerComponent
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsWorkflowEnergyPerComponent
        """
        return super().outputs


class InputsWorkflowEnergyPerComponent(_Inputs):
    """Intermediate class used to connect user inputs to
    workflow_energy_per_component operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.workflow_energy_per_component()
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_energy_type = int()
    >>> op.inputs.energy_type.connect(my_energy_type)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_named_selection1 = str()
    >>> op.inputs.named_selection1.connect(my_named_selection1)
    >>> my_named_selection2 = str()
    >>> op.inputs.named_selection2.connect(my_named_selection2)
    """

    def __init__(self, op: Operator):
        super().__init__(workflow_energy_per_component._spec().inputs, op)
        self._time_scoping = Input(
            workflow_energy_per_component._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(
            workflow_energy_per_component._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mesh_scoping)
        self._energy_type = Input(
            workflow_energy_per_component._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._energy_type)
        self._data_sources = Input(
            workflow_energy_per_component._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)
        self._named_selection1 = Input(
            workflow_energy_per_component._spec().input_pin(5), 5, op, 0
        )
        self._inputs.append(self._named_selection1)
        self._named_selection2 = Input(
            workflow_energy_per_component._spec().input_pin(6), 6, op, 1
        )
        self._inputs.append(self._named_selection2)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Parameters
        ----------
        my_time_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Master scoping. all named selections will be
        intersected with this scoping.

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def energy_type(self):
        """Allows to connect energy_type input to the operator.

        Type of energy to be processed: (0: strain +
        kinetic energy (default), 1: strain
        energy, 2: kinetic energy)

        Parameters
        ----------
        my_energy_type : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> op.inputs.energy_type.connect(my_energy_type)
        >>> # or
        >>> op.inputs.energy_type(my_energy_type)
        """
        return self._energy_type

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def named_selection1(self):
        """Allows to connect named_selection1 input to the operator.

        Named selections. intersection of all  named
        selections with the master scoping
        will be done.

        Parameters
        ----------
        my_named_selection1 : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> op.inputs.named_selection1.connect(my_named_selection1)
        >>> # or
        >>> op.inputs.named_selection1(my_named_selection1)
        """
        return self._named_selection1

    @property
    def named_selection2(self):
        """Allows to connect named_selection2 input to the operator.

        Named selections. intersection of all  named
        selections with the master scoping
        will be done.

        Parameters
        ----------
        my_named_selection2 : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> op.inputs.named_selection2.connect(my_named_selection2)
        >>> # or
        >>> op.inputs.named_selection2(my_named_selection2)
        """
        return self._named_selection2


class OutputsWorkflowEnergyPerComponent(_Outputs):
    """Intermediate class used to get outputs from
    workflow_energy_per_component operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.workflow_energy_per_component()
    >>> # Connect inputs : op.inputs. ...
    >>> result_component_energy = op.outputs.component_energy()
    >>> result_component_energy_percentage = op.outputs.component_energy_percentage()
    """

    def __init__(self, op: Operator):
        super().__init__(workflow_energy_per_component._spec().outputs, op)
        self._component_energy = Output(
            workflow_energy_per_component._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._component_energy)
        self._component_energy_percentage = Output(
            workflow_energy_per_component._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._component_energy_percentage)

    @property
    def component_energy(self):
        """Allows to get component_energy output of the operator

        Returns
        ----------
        my_component_energy : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> # Connect inputs : op.inputs. ...
        >>> result_component_energy = op.outputs.component_energy()
        """  # noqa: E501
        return self._component_energy

    @property
    def component_energy_percentage(self):
        """Allows to get component_energy_percentage output of the operator

        Returns
        ----------
        my_component_energy_percentage : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.workflow_energy_per_component()
        >>> # Connect inputs : op.inputs. ...
        >>> result_component_energy_percentage = op.outputs.component_energy_percentage()
        """  # noqa: E501
        return self._component_energy_percentage



"""
min_max_over_time_by_entity
===========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class min_max_over_time_by_entity(Operator):
    """Evaluates minimum, maximum over time/frequency and returns those min
    max as well as the time/freq where they occurred

    Parameters
    ----------
    fields_container : FieldsContainer
    compute_amplitude : bool, optional
        Do calculate amplitude.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.min_max.min_max_over_time_by_entity()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_compute_amplitude = bool()
    >>> op.inputs.compute_amplitude.connect(my_compute_amplitude)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.min_max.min_max_over_time_by_entity(
    ...     fields_container=my_fields_container,
    ...     compute_amplitude=my_compute_amplitude,
    ... )

    >>> # Get output data
    >>> result_min = op.outputs.min()
    >>> result_max = op.outputs.max()
    >>> result_time_freq_of_min = op.outputs.time_freq_of_min()
    >>> result_time_freq_of_max = op.outputs.time_freq_of_max()
    """

    def __init__(
        self, fields_container=None, compute_amplitude=None, config=None, server=None
    ):
        super().__init__(
            name="min_max_over_time_by_entity", config=config, server=server
        )
        self._inputs = InputsMinMaxOverTimeByEntity(self)
        self._outputs = OutputsMinMaxOverTimeByEntity(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if compute_amplitude is not None:
            self.inputs.compute_amplitude.connect(compute_amplitude)

    @staticmethod
    def _spec():
        description = """Evaluates minimum, maximum over time/frequency and returns those min
            max as well as the time/freq where they occurred"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                4: PinSpecification(
                    name="compute_amplitude",
                    type_names=["bool"],
                    optional=True,
                    document="""Do calculate amplitude.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="min",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="max",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="time_freq_of_min",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                3: PinSpecification(
                    name="time_freq_of_max",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="min_max_over_time_by_entity", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMinMaxOverTimeByEntity
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMinMaxOverTimeByEntity
        """
        return super().outputs


class InputsMinMaxOverTimeByEntity(_Inputs):
    """Intermediate class used to connect user inputs to
    min_max_over_time_by_entity operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_compute_amplitude = bool()
    >>> op.inputs.compute_amplitude.connect(my_compute_amplitude)
    """

    def __init__(self, op: Operator):
        super().__init__(min_max_over_time_by_entity._spec().inputs, op)
        self._fields_container = Input(
            min_max_over_time_by_entity._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._fields_container)
        self._compute_amplitude = Input(
            min_max_over_time_by_entity._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._compute_amplitude)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator.

        Parameters
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def compute_amplitude(self):
        """Allows to connect compute_amplitude input to the operator.

        Do calculate amplitude.

        Parameters
        ----------
        my_compute_amplitude : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
        >>> op.inputs.compute_amplitude.connect(my_compute_amplitude)
        >>> # or
        >>> op.inputs.compute_amplitude(my_compute_amplitude)
        """
        return self._compute_amplitude


class OutputsMinMaxOverTimeByEntity(_Outputs):
    """Intermediate class used to get outputs from
    min_max_over_time_by_entity operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
    >>> # Connect inputs : op.inputs. ...
    >>> result_min = op.outputs.min()
    >>> result_max = op.outputs.max()
    >>> result_time_freq_of_min = op.outputs.time_freq_of_min()
    >>> result_time_freq_of_max = op.outputs.time_freq_of_max()
    """

    def __init__(self, op: Operator):
        super().__init__(min_max_over_time_by_entity._spec().outputs, op)
        self._min = Output(min_max_over_time_by_entity._spec().output_pin(0), 0, op)
        self._outputs.append(self._min)
        self._max = Output(min_max_over_time_by_entity._spec().output_pin(1), 1, op)
        self._outputs.append(self._max)
        self._time_freq_of_min = Output(
            min_max_over_time_by_entity._spec().output_pin(2), 2, op
        )
        self._outputs.append(self._time_freq_of_min)
        self._time_freq_of_max = Output(
            min_max_over_time_by_entity._spec().output_pin(3), 3, op
        )
        self._outputs.append(self._time_freq_of_max)

    @property
    def min(self):
        """Allows to get min output of the operator

        Returns
        ----------
        my_min : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
        >>> # Connect inputs : op.inputs. ...
        >>> result_min = op.outputs.min()
        """  # noqa: E501
        return self._min

    @property
    def max(self):
        """Allows to get max output of the operator

        Returns
        ----------
        my_max : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
        >>> # Connect inputs : op.inputs. ...
        >>> result_max = op.outputs.max()
        """  # noqa: E501
        return self._max

    @property
    def time_freq_of_min(self):
        """Allows to get time_freq_of_min output of the operator

        Returns
        ----------
        my_time_freq_of_min : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
        >>> # Connect inputs : op.inputs. ...
        >>> result_time_freq_of_min = op.outputs.time_freq_of_min()
        """  # noqa: E501
        return self._time_freq_of_min

    @property
    def time_freq_of_max(self):
        """Allows to get time_freq_of_max output of the operator

        Returns
        ----------
        my_time_freq_of_max : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_over_time_by_entity()
        >>> # Connect inputs : op.inputs. ...
        >>> result_time_freq_of_max = op.outputs.time_freq_of_max()
        """  # noqa: E501
        return self._time_freq_of_max



"""
FieldDefinition
================
"""

import traceback
import warnings

from ansys.dpf.core.common import natures, shell_layers
from ansys.dpf.core.check_version import version_requires
from ansys.dpf.core.dimensionality import Dimensionality
from ansys.dpf.core import server as server_module
from ansys.dpf.gate import (
    field_definition_capi,
    field_definition_grpcapi,
    integral_types,
)


class FieldDefinition:
    """Contains the physical and mathematical description of the field.

    Parameters
    ----------
    field_definition : optional
        The default is ``None``.
    server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use
        the global server.
    """

    def __init__(self, field_definition=None, server=None):
        # step 1: get server
        self._server = server_module.get_or_create_server(server)

        # step 2: get api
        self._api = self._server.get_api_for_type(
            capi=field_definition_capi.FieldDefinitionCAPI,
            grpcapi=field_definition_grpcapi.FieldDefinitionGRPCAPI,
        )

        # step3: init environment
        self._api.init_field_definition_environment(self)  # creates stub when gRPC

        # step4: if object exists, take the instance, else create it
        if field_definition is not None:
            self._internal_obj = field_definition
        else:
            if self._server.has_client():
                self._internal_obj = self._api.field_definition_new_on_client(self._server.client)
            else:
                self._internal_obj = self._api.field_definition_new()

    @property
    def location(self):
        """Field location.

        Returns
        -------
        str
            Location string, such as :class:`ansys.dpf.core.locations.nodal`,
            :class:`ansys.dpf.core.locations.elemental` or
            :class:`ansys.dpf.core.locations.time_freq`.
        """
        location = integral_types.MutableString(256)
        size = integral_types.MutableInt32(0)
        self._api.csfield_definition_fill_location(self, location, size)
        return str(location)

    @property
    @version_requires("4.0")
    def name(self):
        """Field name.

        Returns
        -------
        str
        """
        name = integral_types.MutableString(256)
        size = integral_types.MutableInt32(0)
        self._api.csfield_definition_fill_name(self, name, size)
        return str(name)

    @property
    def unit(self):
        """Units of the field.

        Returns
        -------
        str
            Units of the field.
        """
        unit = integral_types.MutableString(256)
        unused = [
            integral_types.MutableInt32(),
            integral_types.MutableInt32(),
            integral_types.MutableDouble(),
            integral_types.MutableDouble(),
        ]
        self._api.csfield_definition_fill_unit(self, unit, *unused)
        return str(unit)

    @property
    def shell_layers(self):
        """Order of the shell layers.

        Returns
        -------
        shell_layers : shell_layers
            ``LayerIndependent`` is returned for fields unrelated to layers.
        """
        enum_val = self._api.csfield_definition_get_shell_layers(self)
        return shell_layers(
            enum_val.real  # - 1
        )  # +1 is added to the proto enum to have notset as 0

    @property
    def dimensionality(self):
        """Dimensionality

        Returns
        -------
        dimensionality : Dimensionality
            Nature and size of the elementary data.
        """
        dim = integral_types.MutableListInt32(size=3)
        nature = integral_types.MutableInt32()
        self._api.csfield_definition_fill_dimensionality(self, dim, nature, dim.internal_size)
        return Dimensionality(dim.tolist(), natures(int(nature)))

    @unit.setter
    def unit(self, value):
        self._api.csfield_definition_set_unit(self, value, None, 0, 0, 0)

    @location.setter
    def location(self, value):
        self._api.csfield_definition_set_location(self, value)

    @name.setter
    @version_requires("4.0")
    def name(self, value):
        self._api.csfield_definition_set_name(self, value)

    @shell_layers.setter
    def shell_layers(self, value):
        if hasattr(value, "value"):
            value = value.value
        self._api.csfield_definition_set_shell_layers(self, value)

    @dimensionality.setter
    def dimensionality(self, value):
        if not isinstance(value, Dimensionality):
            raise TypeError("the dimensionality needs to be of type Dimensionality")
        self._api.csfield_definition_set_dimensionality(
            self, int(value.nature.value), value.dim, len(value.dim)
        )

    def deep_copy(self, server=None):
        """Creates a deep copy of the field_definition's data on a given server.
        This can be useful to pass data from one server instance to another.

        Parameters
        ----------
        server : DPFServer, optional

        Returns
        -------
        field_definition_copy : FieldDefinition
        """
        out = FieldDefinition(server=server)
        out.unit = self.unit
        out.location = self.location
        out.dimensionality = self.dimensionality
        out.shell_layers = self.shell_layers
        return out

    def __del__(self):
        try:
            self._deleter_func[0](self._deleter_func[1](self))
        except:
            warnings.warn(traceback.format_exc())



"""
workflow_import_json
====================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from "serialization" category
"""

class workflow_import_json(Operator):
    """Import a workflow in json format.

      available inputs:
        - json_workflow (str, DataSources)

      available outputs:
        - workflow (Workflow)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.serialization.workflow_import_json()

      >>> # Make input connections
      >>> my_json_workflow = str()
      >>> op.inputs.json_workflow.connect(my_json_workflow)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.serialization.workflow_import_json(json_workflow=my_json_workflow)

      >>> # Get output data
      >>> result_workflow = op.outputs.workflow()"""
    def __init__(self, json_workflow=None, config=None, server=None):
        super().__init__(name="serialization::workflow_import_json", config = config, server = server)
        self._inputs = InputsWorkflowImportJson(self)
        self._outputs = OutputsWorkflowImportJson(self)
        if json_workflow !=None:
            self.inputs.json_workflow.connect(json_workflow)

    @staticmethod
    def _spec():
        spec = Specification(description="""Import a workflow in json format.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "json_workflow", type_names=["string","data_sources"], optional=False, document="""Input json data as either a data source or a string""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "workflow", type_names=["workflow"], optional=False, document="""Instantiate workflow.""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "serialization::workflow_import_json")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator.

        Returns
        --------
        inputs : InputsWorkflowImportJson 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it.

        Returns
        --------
        outputs : OutputsWorkflowImportJson 
        """
        return super().outputs


#internal name: serialization::workflow_import_json
#scripting name: workflow_import_json
class InputsWorkflowImportJson(_Inputs):
    """Intermediate class used to connect user inputs to workflow_import_json operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.workflow_import_json()
      >>> my_json_workflow = str()
      >>> op.inputs.json_workflow.connect(my_json_workflow)
    """
    def __init__(self, op: Operator):
        super().__init__(workflow_import_json._spec().inputs, op)
        self._json_workflow = Input(workflow_import_json._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._json_workflow)

    @property
    def json_workflow(self):
        """Allows to connect json_workflow input to the operator

        - pindoc: Input json data as either a data source or a string

        Parameters
        ----------
        my_json_workflow : str, DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.workflow_import_json()
        >>> op.inputs.json_workflow.connect(my_json_workflow)
        >>> #or
        >>> op.inputs.json_workflow(my_json_workflow)

        """
        return self._json_workflow

class OutputsWorkflowImportJson(_Outputs):
    """Intermediate class used to get outputs from workflow_import_json operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.workflow_import_json()
      >>> # Connect inputs : op.inputs. ...
      >>> result_workflow = op.outputs.workflow()
    """
    def __init__(self, op: Operator):
        super().__init__(workflow_import_json._spec().outputs, op)
        self._workflow = Output(workflow_import_json._spec().output_pin(0), 0, op) 
        self._outputs.append(self._workflow)

    @property
    def workflow(self):
        """Allows to get workflow output of the operator


        - pindoc: Instantiate workflow.

        Returns
        ----------
        my_workflow : Workflow, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.workflow_import_json()
        >>> # Connect inputs : op.inputs. ...
        >>> result_workflow = op.outputs.workflow() 
        """
        return self._workflow




"""
compute_element_centroids
=========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class compute_element_centroids(Operator):
    """Computes the element centroids of the mesh. It also outputs the
    element measure.

    Parameters
    ----------
    element_scoping : Scoping, optional
        If provided, only the centroids of the
        elements in the scoping are computed.
    mesh : MeshedRegion
        Mesh to compute centroids


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.compute_element_centroids()

    >>> # Make input connections
    >>> my_element_scoping = dpf.Scoping()
    >>> op.inputs.element_scoping.connect(my_element_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.compute_element_centroids(
    ...     element_scoping=my_element_scoping,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_centroids = op.outputs.centroids()
    >>> result_measure = op.outputs.measure()
    """

    def __init__(self, element_scoping=None, mesh=None, config=None, server=None):
        super().__init__(name="compute_element_centroids", config=config, server=server)
        self._inputs = InputsComputeElementCentroids(self)
        self._outputs = OutputsComputeElementCentroids(self)
        if element_scoping is not None:
            self.inputs.element_scoping.connect(element_scoping)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Computes the element centroids of the mesh. It also outputs the
            element measure."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="element_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""If provided, only the centroids of the
        elements in the scoping are computed.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""Mesh to compute centroids""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="centroids",
                    type_names=["field"],
                    optional=False,
                    document="""Element centroids.""",
                ),
                1: PinSpecification(
                    name="measure",
                    type_names=["field"],
                    optional=False,
                    document="""Element measure (length, surface or volume
        depending on the dimension of the
        element).""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="compute_element_centroids", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsComputeElementCentroids
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsComputeElementCentroids
        """
        return super().outputs


class InputsComputeElementCentroids(_Inputs):
    """Intermediate class used to connect user inputs to
    compute_element_centroids operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.compute_element_centroids()
    >>> my_element_scoping = dpf.Scoping()
    >>> op.inputs.element_scoping.connect(my_element_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(compute_element_centroids._spec().inputs, op)
        self._element_scoping = Input(
            compute_element_centroids._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._element_scoping)
        self._mesh = Input(compute_element_centroids._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)

    @property
    def element_scoping(self):
        """Allows to connect element_scoping input to the operator.

        If provided, only the centroids of the
        elements in the scoping are computed.

        Parameters
        ----------
        my_element_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.compute_element_centroids()
        >>> op.inputs.element_scoping.connect(my_element_scoping)
        >>> # or
        >>> op.inputs.element_scoping(my_element_scoping)
        """
        return self._element_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Mesh to compute centroids

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.compute_element_centroids()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsComputeElementCentroids(_Outputs):
    """Intermediate class used to get outputs from
    compute_element_centroids operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.compute_element_centroids()
    >>> # Connect inputs : op.inputs. ...
    >>> result_centroids = op.outputs.centroids()
    >>> result_measure = op.outputs.measure()
    """

    def __init__(self, op: Operator):
        super().__init__(compute_element_centroids._spec().outputs, op)
        self._centroids = Output(compute_element_centroids._spec().output_pin(0), 0, op)
        self._outputs.append(self._centroids)
        self._measure = Output(compute_element_centroids._spec().output_pin(1), 1, op)
        self._outputs.append(self._measure)

    @property
    def centroids(self):
        """Allows to get centroids output of the operator

        Returns
        ----------
        my_centroids : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.compute_element_centroids()
        >>> # Connect inputs : op.inputs. ...
        >>> result_centroids = op.outputs.centroids()
        """  # noqa: E501
        return self._centroids

    @property
    def measure(self):
        """Allows to get measure output of the operator

        Returns
        ----------
        my_measure : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.compute_element_centroids()
        >>> # Connect inputs : op.inputs. ...
        >>> result_measure = op.outputs.measure()
        """  # noqa: E501
        return self._measure



"""
Internal Usage
"""
import warnings
import traceback
from ansys.dpf.gate import (
    label_space_capi,
    label_space_grpcapi,
    data_processing_capi,
    data_processing_grpcapi,
)
from ansys.dpf.core import server as server_module


class LabelSpace:
    def __init__(self, label_space=None, obj=None, server=None):
        # ############################
        # step 1: get server
        self._server = server_module.get_or_create_server(server)

        # step 2: get api
        self._api = self._server.get_api_for_type(
            capi=label_space_capi.LabelSpaceCAPI,
            grpcapi=label_space_grpcapi.LabelSpaceGRPCAPI,
        )
        # step3: init environment
        self._api.init_label_space_environment(self)  # creates stub when gRPC

        # step4: if object exists, take the instance, else create it
        if label_space is not None and not isinstance(label_space, dict):
            self._internal_obj = label_space
        else:
            self._internal_obj = self._api.label_space_new_for_object(obj)
            if isinstance(label_space, dict):
                self.fill(label_space)

    @property
    def _data_processing_core_api(self):
        core_api = self._server.get_api_for_type(
            capi=data_processing_capi.DataProcessingCAPI,
            grpcapi=data_processing_grpcapi.DataProcessingGRPCAPI,
        )
        core_api.init_data_processing_environment(self)
        return core_api

    def fill(self, label_space):
        for key, index in label_space.items():
            self._api.label_space_add_data(self, key, index)

    def __dict__(self):
        if isinstance(self._internal_obj, dict):
            return self._internal_obj
        out = {}

        for i in range(0, self._api.label_space_get_size(self)):
            out[
                self._api.label_space_get_labels_name(self, i)
            ] = self._api.label_space_get_labels_value(self, i)
        return out

    def __del__(self):
        try:
            self._deleter_func[0](self._deleter_func[1](self))
        except:
            warnings.warn(traceback.format_exc())



"""
.. _ref_workflow_apis:

Workflow
========
"""
import logging
import traceback
import warnings

from enum import Enum
from ansys import dpf
from ansys.dpf.core import dpf_operator, inputs, outputs
from ansys.dpf.core.check_version import server_meet_version, version_requires
from ansys.dpf.core import server as server_module
from ansys.dpf.gate import (
    workflow_abstract_api,
    workflow_grpcapi,
    workflow_capi,
    data_processing_capi,
    data_processing_grpcapi,
    dpf_vector,
    object_handler,
)

LOG = logging.getLogger(__name__)
LOG.setLevel("DEBUG")


class Workflow:
    """Represents a workflow.

    A workflow is a black box containing operators and exposing only the necessary operator's
    inputs and outputs to compute a given algorithm.

    Parameters
    ----------
    server : DPFServer, optional
        Server with channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    workflow : ctypes.c_void_p, workflow_message_pb2.Workflow, optional

    Examples
    --------
    Create a generic workflow computing the minimum of displacement by chaining the ``'U'``
    and ``'min_max_fc'`` operators.

    >>> from ansys.dpf import core as dpf
    >>> disp_op = dpf.operators.result.displacement()
    >>> max_fc_op = dpf.operators.min_max.min_max_fc(disp_op)
    >>> workflow = dpf.Workflow()
    >>> workflow.add_operators([disp_op,max_fc_op])
    >>> workflow.set_input_name("data_sources", disp_op.inputs.data_sources)
    >>> workflow.set_output_name("min", max_fc_op.outputs.field_min)
    >>> workflow.set_output_name("max", max_fc_op.outputs.field_max)


    >>> from ansys.dpf.core import examples
    >>> data_src = dpf.DataSources(examples.find_multishells_rst())
    >>> workflow.connect("data_sources", data_src)
    >>> min = workflow.get_output("min", dpf.types.field) # doctest: +SKIP
    >>> max = workflow.get_output("max", dpf.types.field) # doctest: +SKIP

    """

    def __init__(self, workflow=None, server=None):
        # step 1: get server
        self._server = server_module.get_or_create_server(server)

        # step 2: get api
        self._api_instance = None  # see property self._api

        # step3: init environment
        self._api.init_workflow_environment(self)  # creates stub when gRPC

        # step4: if object exists, take the instance, else create it
        if workflow is not None:
            self._internal_obj = workflow
        else:
            if self._server.has_client():
                self._internal_obj = self._api.work_flow_new_on_client(self._server.client)
            else:
                self._internal_obj = self._api.work_flow_new()

        self.progress_bar = True

    @property
    def _api(self) -> workflow_abstract_api.WorkflowAbstractAPI:
        if not self._api_instance:
            self._api_instance = self._server.get_api_for_type(
                capi=workflow_capi.WorkflowCAPI,
                grpcapi=workflow_grpcapi.WorkflowGRPCAPI,
            )
        return self._api_instance

    @property
    @version_requires("3.0")
    def progress_bar(self) -> bool:
        """With this property, the user can choose to print a progress bar when
        the workflow's output is requested, default is True"""
        return self._progress_bar

    @progress_bar.setter
    def progress_bar(self, value: bool) -> None:
        self._progress_bar = value

    def connect(self, pin_name, inpt, pin_out=0):
        """Connect an input on the workflow using a pin name.

        Parameters
        ----------
        pin_name : str
            Name of the pin to connect. This name should be
            exposed before with wf.set_input_name
        inpt : str, int, double, bool, list of int, list of doubles,
               Field, FieldsContainer, Scoping, ScopingsContainer,
        MeshedRegion, MeshesContainer, DataSources, Operator
            Object to connect to.
        pin_out : int, optional
            If the input is an operator, the output pin of the input operator.
            The default is ``0``.

        Examples
        --------
        Create a generic workflow computing the minimum of displacement by chaining the ``'U'``
        and ``'min_max_fc'`` operators.

        >>> from ansys.dpf import core as dpf
        >>> disp_op = dpf.operators.result.displacement()
        >>> max_fc_op = dpf.operators.min_max.min_max_fc(disp_op)
        >>> workflow = dpf.Workflow()
        >>> workflow.add_operators([disp_op,max_fc_op])
        >>> workflow.set_input_name("data_sources", disp_op.inputs.data_sources)
        >>> workflow.set_output_name("min", max_fc_op.outputs.field_min)
        >>> workflow.set_output_name("max", max_fc_op.outputs.field_max)


        >>> from ansys.dpf.core import examples
        >>> data_src = dpf.DataSources(examples.find_multishells_rst())
        >>> workflow.connect("data_sources", data_src)
        >>> min = workflow.get_output("min", dpf.types.field) # doctest: +SKIP
        >>> max = workflow.get_output("max", dpf.types.field) # doctest: +SKIP

        """
        if inpt is self:
            raise ValueError("Cannot connect to itself.")
        elif isinstance(inpt, dpf_operator.Operator):
            self._api.work_flow_connect_operator_output(self, pin_name, inpt, pin_out)
        elif isinstance(inpt, dpf_operator.Output):
            self._api.work_flow_connect_operator_output(self, pin_name, inpt._operator, inpt._pin)
        elif isinstance(inpt, list):
            from ansys.dpf.core import collection

            if server_meet_version("3.0", self._server):
                inpt = collection.Collection.integral_collection(inpt, self._server)
                self._api.work_flow_connect_collection_as_vector(self, pin_name, inpt)
            else:
                if all(isinstance(x, int) for x in inpt):
                    self._api.work_flow_connect_vector_int(self, pin_name, inpt, len(inpt))
                else:
                    self._api.work_flow_connect_vector_double(self, pin_name, inpt, len(inpt))
        elif isinstance(inpt, dict):
            from ansys.dpf.core import label_space

            label_space_to_con = label_space.LabelSpace(
                label_space=inpt, obj=self, server=self._server
            )
            self._api.work_flow_connect_label_space(self, pin_name, label_space_to_con)
        else:
            for type_tuple in self._type_to_input_method:
                if isinstance(inpt, type_tuple[0]):
                    if len(type_tuple) == 3:
                        inpt = type_tuple[2](inpt)
                    return type_tuple[1](self, pin_name, inpt)
            errormsg = f"input type {inpt.__class__} cannot be connected"
            raise TypeError(errormsg)

    @property
    def _type_to_input_method(self):
        from ansys.dpf.core import (
            cyclic_support,
            data_sources,
            field,
            collection,
            meshed_region,
            property_field,
            string_field,
            custom_type_field,
            scoping,
            time_freq_support,
            data_tree,
            workflow,
            model,
            generic_data_container,
        )

        out = [
            (bool, self._api.work_flow_connect_bool),
            ((int, Enum), self._api.work_flow_connect_int),
            (str, self._api.work_flow_connect_string),
            (float, self._api.work_flow_connect_double),
            (field.Field, self._api.work_flow_connect_field),
            (property_field.PropertyField, self._api.work_flow_connect_property_field),
            (string_field.StringField, self._api.work_flow_connect_string_field),
            (
                custom_type_field.CustomTypeField,
                self._api.work_flow_connect_custom_type_field,
            ),
            (scoping.Scoping, self._api.work_flow_connect_scoping),
            (collection.Collection, self._api.work_flow_connect_collection),
            (data_sources.DataSources, self._api.work_flow_connect_data_sources),
            (
                model.Model,
                self._api.work_flow_connect_data_sources,
                lambda obj: obj.metadata.data_sources,
            ),
            (cyclic_support.CyclicSupport, self._api.work_flow_connect_cyclic_support),
            (meshed_region.MeshedRegion, self._api.work_flow_connect_meshed_region),
            # TO DO: (result_info.ResultInfo, self._api.work_flow_connect_result_info),
            (
                time_freq_support.TimeFreqSupport,
                self._api.work_flow_connect_time_freq_support,
            ),
            (workflow.Workflow, self._api.work_flow_connect_workflow),
            (data_tree.DataTree, self._api.work_flow_connect_data_tree),
        ]
        if hasattr(self._api, "work_flow_connect_generic_data_container"):
            out.append(
                (
                    generic_data_container.GenericDataContainer,
                    self._api.work_flow_connect_generic_data_container,
                )
            )
        return out

    @property
    def _type_to_output_method(self):
        from ansys.dpf.core import (
            cyclic_support,
            data_sources,
            field,
            fields_container,
            meshed_region,
            meshes_container,
            property_field,
            string_field,
            custom_type_field,
            result_info,
            scoping,
            scopings_container,
            time_freq_support,
            data_tree,
            workflow,
            collection,
            generic_data_container,
        )

        out = [
            (bool, self._api.work_flow_getoutput_bool),
            (int, self._api.work_flow_getoutput_int),
            (str, self._api.work_flow_getoutput_string),
            (float, self._api.work_flow_getoutput_double),
            (field.Field, self._api.work_flow_getoutput_field, "field"),
            (
                property_field.PropertyField,
                self._api.work_flow_getoutput_property_field,
                "property_field",
            ),
            (
                string_field.StringField,
                self._api.work_flow_getoutput_string_field,
                "string_field",
            ),
            (
                custom_type_field.CustomTypeField,
                self._api.work_flow_getoutput_custom_type_field,
                "field",
            ),
            (scoping.Scoping, self._api.work_flow_getoutput_scoping, "scoping"),
            (
                fields_container.FieldsContainer,
                self._api.work_flow_getoutput_fields_container,
                "fields_container",
            ),
            (
                scopings_container.ScopingsContainer,
                self._api.work_flow_getoutput_scopings_container,
                "scopings_container",
            ),
            (
                meshes_container.MeshesContainer,
                self._api.work_flow_getoutput_meshes_container,
                "meshes_container",
            ),
            (
                data_sources.DataSources,
                self._api.work_flow_getoutput_data_sources,
                "data_sources",
            ),
            (
                cyclic_support.CyclicSupport,
                self._api.work_flow_getoutput_cyclic_support,
                "cyclic_support",
            ),
            (
                meshed_region.MeshedRegion,
                self._api.work_flow_getoutput_meshed_region,
                "mesh",
            ),
            (
                result_info.ResultInfo,
                self._api.work_flow_getoutput_result_info,
                "result_info",
            ),
            (
                time_freq_support.TimeFreqSupport,
                self._api.work_flow_getoutput_time_freq_support,
                "time_freq_support",
            ),
            (workflow.Workflow, self._api.work_flow_getoutput_workflow, "workflow"),
            (data_tree.DataTree, self._api.work_flow_getoutput_data_tree, "data_tree"),
            (dpf_operator.Operator, self._api.work_flow_getoutput_operator, "operator"),
            (
                dpf_vector.DPFVectorInt,
                self._api.work_flow_getoutput_int_collection,
                lambda obj: collection.IntCollection(
                    server=self._server, collection=obj
                ).get_integral_entries(),
            ),
            (
                dpf_vector.DPFVectorDouble,
                self._api.work_flow_getoutput_double_collection,
                lambda obj: collection.FloatCollection(
                    server=self._server, collection=obj
                ).get_integral_entries(),
            ),
        ]
        if hasattr(self._api, "work_flow_connect_generic_data_container"):
            out.append(
                (
                    generic_data_container.GenericDataContainer,
                    self._api.work_flow_getoutput_generic_data_container,
                    "generic_data_container",
                )
            )
        return out

    def get_output(self, pin_name, output_type):
        """Retrieve the output of the operator on the pin number.
        A progress bar following the workflow state is printed.

        Parameters
        ----------
        pin_name : str
            Name of the pin to retrieve. This name should be
            exposed before with wf.set_output_name
        output_type : core.type enum
            Type of the requested output.
        """
        if server_meet_version("3.0", self._server) and self.progress_bar:
            # handle progress bar
            self._server.session.add_workflow(self, "workflow")
            self._progress_thread = self._server.session.listen_to_progress()
        output_type = dpf_operator._write_output_type_to_type(output_type)
        out = None
        for type_tuple in self._type_to_output_method:
            if output_type is type_tuple[0]:
                if len(type_tuple) >= 3:
                    if isinstance(type_tuple[2], str):
                        parameters = {type_tuple[2]: type_tuple[1](self, pin_name)}
                        out = output_type(**parameters, server=self._server)
                    else:
                        out = type_tuple[2](type_tuple[1](self, pin_name))
                if out is None:
                    try:
                        out = output_type(type_tuple[1](self, pin_name), server=self._server)
                    except TypeError:
                        self._progress_thread = None
                        out = output_type(type_tuple[1](self, pin_name))
        if out is not None:
            self._progress_thread = None
            return out
        raise TypeError(f"{output_type} is not an implemented Workflow's output")

    def set_input_name(self, name, *args):
        """Set the name of the input pin of the workflow to expose it for future connection.

        Parameters
        ----------
        name : str
            Name of the pin to connect. This name should be
            exposed before with wf.set_input_name
        *args : core.Operator, core.Input, int
            Operator with its input pin number or input to name.

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> workflow = dpf.Workflow()
        >>> disp_op = dpf.operators.result.displacement()
        >>> max_fc_op = dpf.operators.min_max.min_max_fc(disp_op)
        >>> workflow.set_input_name("data_sources", disp_op.inputs.data_sources)

        >>> from ansys.dpf.core import examples
        >>> data_src = dpf.DataSources(examples.find_multishells_rst())
        >>> workflow.connect("data_sources", data_src)

        """
        pin = 0
        operator = None
        for arg in args:
            if isinstance(arg, inputs.Input):
                pin = arg._pin
                operator = arg._operator()
            elif isinstance(arg, dpf_operator.Operator):
                operator = arg
            elif isinstance(arg, int):
                pin = arg
        return self._api.work_flow_set_name_input_pin(self, operator, pin, name)

    def set_output_name(self, name, *args):
        """Set the name of the output pin of the workflow to expose it for future connection.

        Parameters
        ----------
        name : str
            Name of the pin to connect. This name should be
            exposed before with wf.set_input_name
        *args : core.Operator, core.Output, int
            Operator with its outpt pin number or output to name.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> workflow = dpf.Workflow()
        >>> model = dpf.Model(examples.find_simple_bar())
        >>> disp_op = model.results.displacement()
        >>> max_fc_op = dpf.operators.min_max.min_max_fc(disp_op)
        >>> workflow.set_output_name("contour", disp_op.outputs.fields_container)
        >>> fc = workflow.get_output("contour", dpf.types.fields_container) # doctest: +SKIP

        """
        pin = 0
        operator = None
        for arg in args:
            if isinstance(arg, outputs.Output):
                pin = arg._pin
                operator = arg._operator
            elif isinstance(arg, dpf_operator.Operator):
                operator = arg
            elif isinstance(arg, int):
                pin = arg
        return self._api.work_flow_set_name_output_pin(self, operator, pin, name)

    def add_operators(self, operators):
        """Add operators to the list of operators of the workflow.

        Parameters
        ----------
        operators : dpf.core.Operator, list of dpf.core.Operator
            Operators to add to the list.

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> workflow = dpf.Workflow()
        >>> disp_op = dpf.Operator("U")
        >>> max_op = dpf.Operator("min_max")
        >>> workflow.add_operators([disp_op, max_op])

        """
        if isinstance(operators, list):
            for op in operators:
                self.add_operator(op)
        elif isinstance(operators, dpf_operator.Operator):
            self.add_operator(operators)
        else:
            raise TypeError(
                "Operators to add to the workflow are expected to be of "
                f"type {type(list).__name__} or {type(dpf_operator.Operator).__name__}"
            )

    def add_operator(self, operator):
        """Add an operator to the list of operators of the workflow.

        Parameters
        ----------
        operator : dpf.core.Operator
            Operator to add to the list.

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> workflow = dpf.Workflow()
        >>> disp_op = dpf.Operator("U")
        >>> workflow.add_operator(disp_op)

        """
        self._api.work_flow_add_operator(self, operator)

    def record(self, identifier="", transfer_ownership=True):
        """Add the workflow to DPF's internal registry with an ID returned by this method.

        The workflow can be recovered by ``dpf.core.Workflow.get_recorded_workflow(id)``.

        Parameters
        ----------
        identifier : str, optional
            Name given to the workflow.
        transfer_ownership : bool
            Whether to transfer the ownership. The default is ``True``. If the ownership is
            not transferred, the workflow is removed from the internal registry
            as soon as the workflow has been recovered by its ID.

        Returns
        -------
        int

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> workflow = dpf.Workflow()
        >>> disp_op = dpf.Operator("U")
        >>> workflow.add_operator(disp_op)
        >>> # ...
        >>> id = workflow.record()
        >>> workflow_copy = dpf.Workflow.get_recorded_workflow(id)

        """
        return self._api.work_flow_record_instance(self, identifier, transfer_ownership)

    @staticmethod
    def get_recorded_workflow(id, server=None):
        """Retrieve a workflow registered (with workflow.record())

        Parameters
        ----------
        id : int
            ID given by the method "record".

        Returns
        ----------
        workflow : core.Workflow()
            workflow registered in dpf's registry (server side)

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> workflow = dpf.Workflow()
        >>> disp_op = dpf.Operator("U")
        >>> workflow.add_operator(disp_op)
        >>> # ...
        >>> id = workflow.record()
        >>> workflow_copy = dpf.Workflow.get_recorded_workflow(id)

        """
        wf = Workflow(workflow="None", server=server)
        if wf._server.has_client():
            wf._internal_obj = wf._api.work_flow_get_by_identifier_on_client(id, wf._server.client)
        else:
            wf._internal_obj = wf._api.work_flow_get_by_identifier(id)
        if wf._internal_obj is None:
            raise Exception("Unable to get this workflow from the registry")
        return wf

    @property
    def info(self):
        """Dictionary with the operator names and the exposed input and output names.

        Returns
        ----------
        info : dictionarry str->list str
            Dictionary with ``"operator_names"``, ``"input_names"``, and ``"output_names"`` key.
        """
        return {
            "operator_names": self.operator_names,
            "input_names": self.input_names,
            "output_names": self.output_names,
        }

    @property
    def operator_names(self):
        """List of the names of operators added in the workflow.

        Returns
        ----------
        names : list str
        """
        num = self._api.work_flow_number_of_operators(self)
        out = []
        for i in range(num):
            out.append(self._api.work_flow_operator_name_by_index(self, i))
        return out

    @property
    def input_names(self):
        """List of the input names exposed in the workflow with set_input_name.

        Returns
        ----------
        names : list str
        """
        num = self._api.work_flow_number_of_input(self)
        out = []
        for i in range(num):
            out.append(self._api.work_flow_input_by_index(self, i))
        return out

    @property
    def output_names(self):
        """List of the output names exposed in the workflow with set_output_name.

        Returns
        ----------
        names : list str
        """
        num = self._api.work_flow_number_of_output(self)
        out = []
        for i in range(num):
            out.append(self._api.work_flow_output_by_index(self, i))
        return out

    @version_requires("3.0")
    def connect_with(self, left_workflow, output_input_names=None):
        """Chain 2 workflows together so that they become one workflow.

        The one workflow contains all the operators, inputs, and outputs
        exposed in both workflows.

        Parameters
        ----------
        left_workflow : core.Workflow
            Second workflow's outputs to chained with this workflow's inputs.
        output_input_names : str tuple, str dict optional
            Input name of the left_workflow to be cained with the output name of this workflow.
            The default is ``None``, in which case the inputs in the left_workflow with the same
            names as the outputs of this workflow are chained.

        Examples
        --------
        ::

            +-------------------------------------------------------------------------------------------------+
            |  INPUT:                                                                                         |
            |                                                                                                 |
            |input_output_names = ("output","field" )                                                         |
            |                      _____________                                  ____________                |
            |  "data_sources"  -> |left_workflow| ->  "stuff"        "field" -> |     this   | -> "contour"   |
            |"time_scoping"    -> |             |             "mesh_scoping" -> |            |                |
            |                     |_____________| ->  "output"                  |____________|                |
            |  OUTPUT                                                                                         |
            |                    ____                                                                         |
            |"data_sources"  -> |this| ->  "stuff"                                                            |
            |"time_scoping" ->  |    | ->  "contour"                                                          |
            |"mesh_scoping" ->  |____| -> "output"                                                            |
            +-------------------------------------------------------------------------------------------------+ # noqa: E501

        Notes
        -----
        Function available with server's version starting at 3.0.

        """
        if output_input_names:
            core_api = self._server.get_api_for_type(
                capi=data_processing_capi.DataProcessingCAPI,
                grpcapi=data_processing_grpcapi.DataProcessingGRPCAPI,
            )
            map = object_handler.ObjHandler(
                data_processing_api=core_api,
                internal_obj=self._api.workflow_create_connection_map_for_object(self),
            )
            if isinstance(output_input_names, tuple):
                self._api.workflow_add_entry_connection_map(
                    map, output_input_names[0], output_input_names[1]
                )
            elif isinstance(output_input_names, dict):
                for key in output_input_names:
                    self._api.workflow_add_entry_connection_map(map, key, output_input_names[key])
            else:
                raise TypeError(
                    "output_input_names argument is expect" "to be either a str tuple or a str dict"
                )
            self._api.work_flow_connect_with_specified_names(self, left_workflow, map)
        else:
            self._api.work_flow_connect_with(self, left_workflow)

    @version_requires("3.0")
    def create_on_other_server(self, *args, **kwargs):
        """Create a new instance of a workflow on another server. The new
        Workflow has the same operators, exposed inputs and output pins as
        this workflow. Connections between operators and between data and
        operators are kept (except for exposed pins).

        Parameters
        ----------
        server : server.LegacyGrpcServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        ip : str, optional
            ip address on which the new instance should be created (always put
            a port in args as well)

        port : str, int , optional

        address: str, optional
            address on which the new instance should be created ("ip:port")

        Returns
        -------
        Workflow

        Examples
        --------
        Create a generic Workflow computing the minimum of displacement by chaining the ``'U'``
        and ``'min_max_fc'`` operators.

        >>> from ansys.dpf import core as dpf
        >>> disp_op = dpf.operators.result.displacement()
        >>> max_fc_op = dpf.operators.min_max.min_max_fc(disp_op)
        >>> workflow = dpf.Workflow()
        >>> workflow.add_operators([disp_op,max_fc_op])
        >>> workflow.set_input_name("data_sources", disp_op.inputs.data_sources)
        >>> workflow.set_output_name("min", max_fc_op.outputs.field_min)
        >>> workflow.set_output_name("max", max_fc_op.outputs.field_max)
        >>> #other_server = dpf.start_local_server(as_global=False)
        >>> #new_workflow = workflow.create_on_other_server(server=other_server)
        >>> #assert 'data_sources' in new_workflow.input_names

        """
        server = None
        address = None
        for arg in args:
            if isinstance(arg, dpf.core.server_types.BaseServer):
                server = arg
            elif isinstance(arg, str):
                address = arg

        if "ip" in kwargs:
            address = kwargs["ip"] + ":" + str(kwargs["port"])
        if "address" in kwargs:
            address = kwargs["address"]
        if "server" in kwargs:
            server = kwargs["server"]
        if server:
            text_stream = self._api.work_flow_write_to_text(self)
            wf = Workflow(workflow="None", server=server)
            if wf._server.has_client():
                wf._internal_obj = wf._api.work_flow_create_from_text_on_client(
                    text_stream, wf._server.client
                )
            else:
                wf._internal_obj = wf._api.work_flow_create_from_text(text_stream)
            return wf
        elif address:
            internal_obj = self._api.work_flow_get_copy_on_other_client(self, address, "grpc")
            return Workflow(workflow=internal_obj, server=self._server)
        else:
            raise ValueError(
                "a connection address (either with address input"
                "or both ip and port inputs) or a server is required"
            )

    def __del__(self):
        try:
            if hasattr(self, "_internal_obj"):
                if self._internal_obj is not None and self._internal_obj != "None":
                    self._deleter_func[0](self._deleter_func[1](self))
        except:
            warnings.warn(traceback.format_exc())

    def __str__(self):
        """Describe the entity.

        Returns
        -------
        description : str
        """
        from ansys.dpf.core.core import _description

        return _description(self._internal_obj, self._server)



"""
make_time_chunk_for_each
========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class make_time_chunk_for_each(Operator):
    """Splits a list of time/freq values into chunks depending on evaluated
    result properties,mesh size and max number of bytes accepted and
    calls 'make_for_each_range' to generate a range that can be
    consumed by the for_each operator

    Parameters
    ----------
    target_time_freq_values : float
        List of time/freq values to potentially split
        in chunks.
    operator_to_iterate : Operator
        Operator that must be reconnected with the
        range values.
    pin_index : int
    abstract_meshed_region : MeshedRegion
        The number of nodes (for "nodal" results) or
        number of elements (for "elemental"
        results) is used to compute the
        chunk.
    chunk_config : DataTree
        A data tree with an int attribute
        "max_num_bytes", an int attribute
        "dimensionality" (average result size
        by entity), a string attribute
        "location" ("nodal" or"elemental") is
        expected.
    producer_op11 : Operator
    producer_op12 : Operator
    output_pin_of_producer_op11 : int
    output_pin_of_producer_op12 : int
    input_pin_of_consumer_op11 : int
    input_pin_of_consumer_op12 : int
    consumer_op11 : Operator
    consumer_op12 : Operator


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.make_time_chunk_for_each()

    >>> # Make input connections
    >>> my_target_time_freq_values = float()
    >>> op.inputs.target_time_freq_values.connect(my_target_time_freq_values)
    >>> my_operator_to_iterate = dpf.Operator()
    >>> op.inputs.operator_to_iterate.connect(my_operator_to_iterate)
    >>> my_pin_index = int()
    >>> op.inputs.pin_index.connect(my_pin_index)
    >>> my_abstract_meshed_region = dpf.MeshedRegion()
    >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
    >>> my_chunk_config = dpf.DataTree()
    >>> op.inputs.chunk_config.connect(my_chunk_config)
    >>> my_producer_op11 = dpf.Operator()
    >>> op.inputs.producer_op11.connect(my_producer_op11)
    >>> my_producer_op12 = dpf.Operator()
    >>> op.inputs.producer_op12.connect(my_producer_op12)
    >>> my_output_pin_of_producer_op11 = int()
    >>> op.inputs.output_pin_of_producer_op11.connect(my_output_pin_of_producer_op11)
    >>> my_output_pin_of_producer_op12 = int()
    >>> op.inputs.output_pin_of_producer_op12.connect(my_output_pin_of_producer_op12)
    >>> my_input_pin_of_consumer_op11 = int()
    >>> op.inputs.input_pin_of_consumer_op11.connect(my_input_pin_of_consumer_op11)
    >>> my_input_pin_of_consumer_op12 = int()
    >>> op.inputs.input_pin_of_consumer_op12.connect(my_input_pin_of_consumer_op12)
    >>> my_consumer_op11 = dpf.Operator()
    >>> op.inputs.consumer_op11.connect(my_consumer_op11)
    >>> my_consumer_op12 = dpf.Operator()
    >>> op.inputs.consumer_op12.connect(my_consumer_op12)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.make_time_chunk_for_each(
    ...     target_time_freq_values=my_target_time_freq_values,
    ...     operator_to_iterate=my_operator_to_iterate,
    ...     pin_index=my_pin_index,
    ...     abstract_meshed_region=my_abstract_meshed_region,
    ...     chunk_config=my_chunk_config,
    ...     producer_op11=my_producer_op11,
    ...     producer_op12=my_producer_op12,
    ...     output_pin_of_producer_op11=my_output_pin_of_producer_op11,
    ...     output_pin_of_producer_op12=my_output_pin_of_producer_op12,
    ...     input_pin_of_consumer_op11=my_input_pin_of_consumer_op11,
    ...     input_pin_of_consumer_op12=my_input_pin_of_consumer_op12,
    ...     consumer_op11=my_consumer_op11,
    ...     consumer_op12=my_consumer_op12,
    ... )

    >>> # Get output data
    >>> result_chunks = op.outputs.chunks()
    """

    def __init__(
        self,
        target_time_freq_values=None,
        operator_to_iterate=None,
        pin_index=None,
        abstract_meshed_region=None,
        chunk_config=None,
        producer_op11=None,
        producer_op12=None,
        output_pin_of_producer_op11=None,
        output_pin_of_producer_op12=None,
        input_pin_of_consumer_op11=None,
        input_pin_of_consumer_op12=None,
        consumer_op11=None,
        consumer_op12=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="mechanical::make_time_chunk_for_each", config=config, server=server
        )
        self._inputs = InputsMakeTimeChunkForEach(self)
        self._outputs = OutputsMakeTimeChunkForEach(self)
        if target_time_freq_values is not None:
            self.inputs.target_time_freq_values.connect(target_time_freq_values)
        if operator_to_iterate is not None:
            self.inputs.operator_to_iterate.connect(operator_to_iterate)
        if pin_index is not None:
            self.inputs.pin_index.connect(pin_index)
        if abstract_meshed_region is not None:
            self.inputs.abstract_meshed_region.connect(abstract_meshed_region)
        if chunk_config is not None:
            self.inputs.chunk_config.connect(chunk_config)
        if producer_op11 is not None:
            self.inputs.producer_op11.connect(producer_op11)
        if producer_op12 is not None:
            self.inputs.producer_op12.connect(producer_op12)
        if output_pin_of_producer_op11 is not None:
            self.inputs.output_pin_of_producer_op11.connect(output_pin_of_producer_op11)
        if output_pin_of_producer_op12 is not None:
            self.inputs.output_pin_of_producer_op12.connect(output_pin_of_producer_op12)
        if input_pin_of_consumer_op11 is not None:
            self.inputs.input_pin_of_consumer_op11.connect(input_pin_of_consumer_op11)
        if input_pin_of_consumer_op12 is not None:
            self.inputs.input_pin_of_consumer_op12.connect(input_pin_of_consumer_op12)
        if consumer_op11 is not None:
            self.inputs.consumer_op11.connect(consumer_op11)
        if consumer_op12 is not None:
            self.inputs.consumer_op12.connect(consumer_op12)

    @staticmethod
    def _spec():
        description = """Splits a list of time/freq values into chunks depending on evaluated
            result properties,mesh size and max number of bytes
            accepted and calls &quot;make_for_each_range&quot; to generate a
            range that can be consumed by the for_each operator"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="target_time_freq_values",
                    type_names=["vector<double>", "double"],
                    optional=False,
                    document="""List of time/freq values to potentially split
        in chunks.""",
                ),
                1: PinSpecification(
                    name="operator_to_iterate",
                    type_names=["operator"],
                    optional=False,
                    document="""Operator that must be reconnected with the
        range values.""",
                ),
                2: PinSpecification(
                    name="pin_index",
                    type_names=["int32"],
                    optional=False,
                    document="""""",
                ),
                7: PinSpecification(
                    name="abstract_meshed_region",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""The number of nodes (for "nodal" results) or
        number of elements (for "elemental"
        results) is used to compute the
        chunk.""",
                ),
                200: PinSpecification(
                    name="chunk_config",
                    type_names=["abstract_data_tree"],
                    optional=False,
                    document="""A data tree with an int attribute
        "max_num_bytes", an int attribute
        "dimensionality" (average result size
        by entity), a string attribute
        "location" ("nodal" or"elemental") is
        expected.""",
                ),
                1000: PinSpecification(
                    name="producer_op1",
                    type_names=["operator"],
                    optional=False,
                    document="""""",
                ),
                1001: PinSpecification(
                    name="producer_op1",
                    type_names=["operator"],
                    optional=False,
                    document="""""",
                ),
                1001: PinSpecification(
                    name="output_pin_of_producer_op1",
                    type_names=["int32"],
                    optional=False,
                    document="""""",
                ),
                1002: PinSpecification(
                    name="output_pin_of_producer_op1",
                    type_names=["int32"],
                    optional=False,
                    document="""""",
                ),
                1002: PinSpecification(
                    name="input_pin_of_consumer_op1",
                    type_names=["int32"],
                    optional=False,
                    document="""""",
                ),
                1003: PinSpecification(
                    name="input_pin_of_consumer_op1",
                    type_names=["int32"],
                    optional=False,
                    document="""""",
                ),
                1003: PinSpecification(
                    name="consumer_op1",
                    type_names=["operator"],
                    optional=False,
                    document="""""",
                ),
                1004: PinSpecification(
                    name="consumer_op1",
                    type_names=["operator"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="chunks",
                    optional=False,
                    document="""To connect to "producer_consumer_for_each"
        operator on pin 0""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="mechanical::make_time_chunk_for_each", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMakeTimeChunkForEach
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMakeTimeChunkForEach
        """
        return super().outputs


class InputsMakeTimeChunkForEach(_Inputs):
    """Intermediate class used to connect user inputs to
    make_time_chunk_for_each operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.make_time_chunk_for_each()
    >>> my_target_time_freq_values = float()
    >>> op.inputs.target_time_freq_values.connect(my_target_time_freq_values)
    >>> my_operator_to_iterate = dpf.Operator()
    >>> op.inputs.operator_to_iterate.connect(my_operator_to_iterate)
    >>> my_pin_index = int()
    >>> op.inputs.pin_index.connect(my_pin_index)
    >>> my_abstract_meshed_region = dpf.MeshedRegion()
    >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
    >>> my_chunk_config = dpf.DataTree()
    >>> op.inputs.chunk_config.connect(my_chunk_config)
    >>> my_producer_op11 = dpf.Operator()
    >>> op.inputs.producer_op11.connect(my_producer_op11)
    >>> my_producer_op12 = dpf.Operator()
    >>> op.inputs.producer_op12.connect(my_producer_op12)
    >>> my_output_pin_of_producer_op11 = int()
    >>> op.inputs.output_pin_of_producer_op11.connect(my_output_pin_of_producer_op11)
    >>> my_output_pin_of_producer_op12 = int()
    >>> op.inputs.output_pin_of_producer_op12.connect(my_output_pin_of_producer_op12)
    >>> my_input_pin_of_consumer_op11 = int()
    >>> op.inputs.input_pin_of_consumer_op11.connect(my_input_pin_of_consumer_op11)
    >>> my_input_pin_of_consumer_op12 = int()
    >>> op.inputs.input_pin_of_consumer_op12.connect(my_input_pin_of_consumer_op12)
    >>> my_consumer_op11 = dpf.Operator()
    >>> op.inputs.consumer_op11.connect(my_consumer_op11)
    >>> my_consumer_op12 = dpf.Operator()
    >>> op.inputs.consumer_op12.connect(my_consumer_op12)
    """

    def __init__(self, op: Operator):
        super().__init__(make_time_chunk_for_each._spec().inputs, op)
        self._target_time_freq_values = Input(
            make_time_chunk_for_each._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._target_time_freq_values)
        self._operator_to_iterate = Input(
            make_time_chunk_for_each._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._operator_to_iterate)
        self._pin_index = Input(
            make_time_chunk_for_each._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._pin_index)
        self._abstract_meshed_region = Input(
            make_time_chunk_for_each._spec().input_pin(7), 7, op, -1
        )
        self._inputs.append(self._abstract_meshed_region)
        self._chunk_config = Input(
            make_time_chunk_for_each._spec().input_pin(200), 200, op, -1
        )
        self._inputs.append(self._chunk_config)
        self._producer_op11 = Input(
            make_time_chunk_for_each._spec().input_pin(1000), 1000, op, 0
        )
        self._inputs.append(self._producer_op11)
        self._producer_op12 = Input(
            make_time_chunk_for_each._spec().input_pin(1001), 1001, op, 1
        )
        self._inputs.append(self._producer_op12)
        self._output_pin_of_producer_op11 = Input(
            make_time_chunk_for_each._spec().input_pin(1001), 1001, op, 0
        )
        self._inputs.append(self._output_pin_of_producer_op11)
        self._output_pin_of_producer_op12 = Input(
            make_time_chunk_for_each._spec().input_pin(1002), 1002, op, 1
        )
        self._inputs.append(self._output_pin_of_producer_op12)
        self._input_pin_of_consumer_op11 = Input(
            make_time_chunk_for_each._spec().input_pin(1002), 1002, op, 0
        )
        self._inputs.append(self._input_pin_of_consumer_op11)
        self._input_pin_of_consumer_op12 = Input(
            make_time_chunk_for_each._spec().input_pin(1003), 1003, op, 1
        )
        self._inputs.append(self._input_pin_of_consumer_op12)
        self._consumer_op11 = Input(
            make_time_chunk_for_each._spec().input_pin(1003), 1003, op, 0
        )
        self._inputs.append(self._consumer_op11)
        self._consumer_op12 = Input(
            make_time_chunk_for_each._spec().input_pin(1004), 1004, op, 1
        )
        self._inputs.append(self._consumer_op12)

    @property
    def target_time_freq_values(self):
        """Allows to connect target_time_freq_values input to the operator.

        List of time/freq values to potentially split
        in chunks.

        Parameters
        ----------
        my_target_time_freq_values : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.target_time_freq_values.connect(my_target_time_freq_values)
        >>> # or
        >>> op.inputs.target_time_freq_values(my_target_time_freq_values)
        """
        return self._target_time_freq_values

    @property
    def operator_to_iterate(self):
        """Allows to connect operator_to_iterate input to the operator.

        Operator that must be reconnected with the
        range values.

        Parameters
        ----------
        my_operator_to_iterate : Operator

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.operator_to_iterate.connect(my_operator_to_iterate)
        >>> # or
        >>> op.inputs.operator_to_iterate(my_operator_to_iterate)
        """
        return self._operator_to_iterate

    @property
    def pin_index(self):
        """Allows to connect pin_index input to the operator.

        Parameters
        ----------
        my_pin_index : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.pin_index.connect(my_pin_index)
        >>> # or
        >>> op.inputs.pin_index(my_pin_index)
        """
        return self._pin_index

    @property
    def abstract_meshed_region(self):
        """Allows to connect abstract_meshed_region input to the operator.

        The number of nodes (for "nodal" results) or
        number of elements (for "elemental"
        results) is used to compute the
        chunk.

        Parameters
        ----------
        my_abstract_meshed_region : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
        >>> # or
        >>> op.inputs.abstract_meshed_region(my_abstract_meshed_region)
        """
        return self._abstract_meshed_region

    @property
    def chunk_config(self):
        """Allows to connect chunk_config input to the operator.

        A data tree with an int attribute
        "max_num_bytes", an int attribute
        "dimensionality" (average result size
        by entity), a string attribute
        "location" ("nodal" or"elemental") is
        expected.

        Parameters
        ----------
        my_chunk_config : DataTree

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.chunk_config.connect(my_chunk_config)
        >>> # or
        >>> op.inputs.chunk_config(my_chunk_config)
        """
        return self._chunk_config

    @property
    def producer_op11(self):
        """Allows to connect producer_op11 input to the operator.

        Parameters
        ----------
        my_producer_op11 : Operator

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.producer_op11.connect(my_producer_op11)
        >>> # or
        >>> op.inputs.producer_op11(my_producer_op11)
        """
        return self._producer_op11

    @property
    def producer_op12(self):
        """Allows to connect producer_op12 input to the operator.

        Parameters
        ----------
        my_producer_op12 : Operator

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.producer_op12.connect(my_producer_op12)
        >>> # or
        >>> op.inputs.producer_op12(my_producer_op12)
        """
        return self._producer_op12

    @property
    def output_pin_of_producer_op11(self):
        """Allows to connect output_pin_of_producer_op11 input to the operator.

        Parameters
        ----------
        my_output_pin_of_producer_op11 : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.output_pin_of_producer_op11.connect(my_output_pin_of_producer_op11)
        >>> # or
        >>> op.inputs.output_pin_of_producer_op11(my_output_pin_of_producer_op11)
        """
        return self._output_pin_of_producer_op11

    @property
    def output_pin_of_producer_op12(self):
        """Allows to connect output_pin_of_producer_op12 input to the operator.

        Parameters
        ----------
        my_output_pin_of_producer_op12 : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.output_pin_of_producer_op12.connect(my_output_pin_of_producer_op12)
        >>> # or
        >>> op.inputs.output_pin_of_producer_op12(my_output_pin_of_producer_op12)
        """
        return self._output_pin_of_producer_op12

    @property
    def input_pin_of_consumer_op11(self):
        """Allows to connect input_pin_of_consumer_op11 input to the operator.

        Parameters
        ----------
        my_input_pin_of_consumer_op11 : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.input_pin_of_consumer_op11.connect(my_input_pin_of_consumer_op11)
        >>> # or
        >>> op.inputs.input_pin_of_consumer_op11(my_input_pin_of_consumer_op11)
        """
        return self._input_pin_of_consumer_op11

    @property
    def input_pin_of_consumer_op12(self):
        """Allows to connect input_pin_of_consumer_op12 input to the operator.

        Parameters
        ----------
        my_input_pin_of_consumer_op12 : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.input_pin_of_consumer_op12.connect(my_input_pin_of_consumer_op12)
        >>> # or
        >>> op.inputs.input_pin_of_consumer_op12(my_input_pin_of_consumer_op12)
        """
        return self._input_pin_of_consumer_op12

    @property
    def consumer_op11(self):
        """Allows to connect consumer_op11 input to the operator.

        Parameters
        ----------
        my_consumer_op11 : Operator

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.consumer_op11.connect(my_consumer_op11)
        >>> # or
        >>> op.inputs.consumer_op11(my_consumer_op11)
        """
        return self._consumer_op11

    @property
    def consumer_op12(self):
        """Allows to connect consumer_op12 input to the operator.

        Parameters
        ----------
        my_consumer_op12 : Operator

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> op.inputs.consumer_op12.connect(my_consumer_op12)
        >>> # or
        >>> op.inputs.consumer_op12(my_consumer_op12)
        """
        return self._consumer_op12


class OutputsMakeTimeChunkForEach(_Outputs):
    """Intermediate class used to get outputs from
    make_time_chunk_for_each operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.make_time_chunk_for_each()
    >>> # Connect inputs : op.inputs. ...
    >>> result_chunks = op.outputs.chunks()
    """

    def __init__(self, op: Operator):
        super().__init__(make_time_chunk_for_each._spec().outputs, op)
        self._chunks = Output(make_time_chunk_for_each._spec().output_pin(0), 0, op)
        self._outputs.append(self._chunks)

    @property
    def chunks(self):
        """Allows to get chunks output of the operator

        Returns
        ----------
        my_chunks :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_time_chunk_for_each()
        >>> # Connect inputs : op.inputs. ...
        >>> result_chunks = op.outputs.chunks()
        """  # noqa: E501
        return self._chunks



"""
mesh_provider
=============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mesh_provider(Operator):
    """Reads a mesh from result files.

    Parameters
    ----------
    time_scoping : int, optional
        Optional time/frequency set id of the mesh,
        supported for adaptative meshes.
    streams_container : StreamsContainer, optional
        Result file container allowed to be kept open
        to cache data
    data_sources : DataSources
        Result file path container, used if no
        streams are set
    read_cyclic : int, optional
        If 1, cyclic symmetry is ignored. if 2,
        cyclic expansion is done (default is
        1).
    region_scoping : Scoping or int, optional
        Region id (integer) or vector of region ids
        with one entity (vector) or region
        scoping with one id (scoping) (region
        corresponds to zone for fluid results
        or part for lsdyna results).
    laziness : DataTree, optional
        Configurate whether lazy evaluation can be
        performed and to what extent.
        supported attributes are:  -
        "num_named_selections"->num named
        selection to read (-1 is all, int32,
        default si -1), careful: the other
        named selections will not be
        available, use mesh_property_provider
        operator to read them. - all mesh
        property fields "mat",
        "named_selection",
        "apdl_element_type", "section"-> if
        set to 1 these properties will not be
        read and a workflow will be bounded
        to the properties to be evaluated on
        demand, with 0 they are read (default
        is 0).


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.mesh_provider()

    >>> # Make input connections
    >>> my_time_scoping = int()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_read_cyclic = int()
    >>> op.inputs.read_cyclic.connect(my_read_cyclic)
    >>> my_region_scoping = dpf.Scoping()
    >>> op.inputs.region_scoping.connect(my_region_scoping)
    >>> my_laziness = dpf.DataTree()
    >>> op.inputs.laziness.connect(my_laziness)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.mesh_provider(
    ...     time_scoping=my_time_scoping,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ...     read_cyclic=my_read_cyclic,
    ...     region_scoping=my_region_scoping,
    ...     laziness=my_laziness,
    ... )

    >>> # Get output data
    >>> result_mesh = op.outputs.mesh()
    """

    def __init__(
        self,
        time_scoping=None,
        streams_container=None,
        data_sources=None,
        read_cyclic=None,
        region_scoping=None,
        laziness=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mesh_provider", config=config, server=server)
        self._inputs = InputsMeshProvider(self)
        self._outputs = OutputsMeshProvider(self)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if read_cyclic is not None:
            self.inputs.read_cyclic.connect(read_cyclic)
        if region_scoping is not None:
            self.inputs.region_scoping.connect(region_scoping)
        if laziness is not None:
            self.inputs.laziness.connect(laziness)

    @staticmethod
    def _spec():
        description = """Reads a mesh from result files."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_scoping",
                    type_names=["int32"],
                    optional=True,
                    document="""Optional time/frequency set id of the mesh,
        supported for adaptative meshes.""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Result file container allowed to be kept open
        to cache data""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Result file path container, used if no
        streams are set""",
                ),
                14: PinSpecification(
                    name="read_cyclic",
                    type_names=["enum dataProcessing::ECyclicReading", "int32"],
                    optional=True,
                    document="""If 1, cyclic symmetry is ignored. if 2,
        cyclic expansion is done (default is
        1).""",
                ),
                25: PinSpecification(
                    name="region_scoping",
                    type_names=["scoping", "int32", "vector<int32>"],
                    optional=True,
                    document="""Region id (integer) or vector of region ids
        with one entity (vector) or region
        scoping with one id (scoping) (region
        corresponds to zone for fluid results
        or part for lsdyna results).""",
                ),
                200: PinSpecification(
                    name="laziness",
                    type_names=["abstract_data_tree"],
                    optional=True,
                    document="""Configurate whether lazy evaluation can be
        performed and to what extent.
        supported attributes are:  -
        "num_named_selections"->num named
        selection to read (-1 is all, int32,
        default si -1), careful: the other
        named selections will not be
        available, use mesh_property_provider
        operator to read them. - all mesh
        property fields "mat",
        "named_selection",
        "apdl_element_type", "section"-> if
        set to 1 these properties will not be
        read and a workflow will be bounded
        to the properties to be evaluated on
        demand, with 0 they are read (default
        is 0).""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mesh_provider", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshProvider
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMeshProvider
        """
        return super().outputs


class InputsMeshProvider(_Inputs):
    """Intermediate class used to connect user inputs to
    mesh_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_provider()
    >>> my_time_scoping = int()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_read_cyclic = int()
    >>> op.inputs.read_cyclic.connect(my_read_cyclic)
    >>> my_region_scoping = dpf.Scoping()
    >>> op.inputs.region_scoping.connect(my_region_scoping)
    >>> my_laziness = dpf.DataTree()
    >>> op.inputs.laziness.connect(my_laziness)
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_provider._spec().inputs, op)
        self._time_scoping = Input(mesh_provider._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._time_scoping)
        self._streams_container = Input(mesh_provider._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._streams_container)
        self._data_sources = Input(mesh_provider._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)
        self._read_cyclic = Input(mesh_provider._spec().input_pin(14), 14, op, -1)
        self._inputs.append(self._read_cyclic)
        self._region_scoping = Input(mesh_provider._spec().input_pin(25), 25, op, -1)
        self._inputs.append(self._region_scoping)
        self._laziness = Input(mesh_provider._spec().input_pin(200), 200, op, -1)
        self._inputs.append(self._laziness)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Optional time/frequency set id of the mesh,
        supported for adaptative meshes.

        Parameters
        ----------
        my_time_scoping : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_provider()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator.

        Result file container allowed to be kept open
        to cache data

        Parameters
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_provider()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Result file path container, used if no
        streams are set

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def read_cyclic(self):
        """Allows to connect read_cyclic input to the operator.

        If 1, cyclic symmetry is ignored. if 2,
        cyclic expansion is done (default is
        1).

        Parameters
        ----------
        my_read_cyclic : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_provider()
        >>> op.inputs.read_cyclic.connect(my_read_cyclic)
        >>> # or
        >>> op.inputs.read_cyclic(my_read_cyclic)
        """
        return self._read_cyclic

    @property
    def region_scoping(self):
        """Allows to connect region_scoping input to the operator.

        Region id (integer) or vector of region ids
        with one entity (vector) or region
        scoping with one id (scoping) (region
        corresponds to zone for fluid results
        or part for lsdyna results).

        Parameters
        ----------
        my_region_scoping : Scoping or int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_provider()
        >>> op.inputs.region_scoping.connect(my_region_scoping)
        >>> # or
        >>> op.inputs.region_scoping(my_region_scoping)
        """
        return self._region_scoping

    @property
    def laziness(self):
        """Allows to connect laziness input to the operator.

        Configurate whether lazy evaluation can be
        performed and to what extent.
        supported attributes are:  -
        "num_named_selections"->num named
        selection to read (-1 is all, int32,
        default si -1), careful: the other
        named selections will not be
        available, use mesh_property_provider
        operator to read them. - all mesh
        property fields "mat",
        "named_selection",
        "apdl_element_type", "section"-> if
        set to 1 these properties will not be
        read and a workflow will be bounded
        to the properties to be evaluated on
        demand, with 0 they are read (default
        is 0).

        Parameters
        ----------
        my_laziness : DataTree

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_provider()
        >>> op.inputs.laziness.connect(my_laziness)
        >>> # or
        >>> op.inputs.laziness(my_laziness)
        """
        return self._laziness


class OutputsMeshProvider(_Outputs):
    """Intermediate class used to get outputs from
    mesh_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_provider()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mesh = op.outputs.mesh()
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_provider._spec().outputs, op)
        self._mesh = Output(mesh_provider._spec().output_pin(0), 0, op)
        self._outputs.append(self._mesh)

    @property
    def mesh(self):
        """Allows to get mesh output of the operator

        Returns
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_provider()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh = op.outputs.mesh()
        """  # noqa: E501
        return self._mesh



"""
modal_participation
===================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class modal_participation(Operator):
    """Compute the modal participation factor for a given vector field V,
    defined as  sum_i ( V .dot. mode_shape_i * ponderation ).

    Parameters
    ----------
    v_real : Field
        Real part of field v
    v_imag : Field
        Imag part of field v
    mode_shapes : FieldsContainer
    ponderation : Field, optional
    force_label_space : dict, optional
        If set, will force a label space for output
        result.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.modal_participation()

    >>> # Make input connections
    >>> my_v_real = dpf.Field()
    >>> op.inputs.v_real.connect(my_v_real)
    >>> my_v_imag = dpf.Field()
    >>> op.inputs.v_imag.connect(my_v_imag)
    >>> my_mode_shapes = dpf.FieldsContainer()
    >>> op.inputs.mode_shapes.connect(my_mode_shapes)
    >>> my_ponderation = dpf.Field()
    >>> op.inputs.ponderation.connect(my_ponderation)
    >>> my_force_label_space = dict()
    >>> op.inputs.force_label_space.connect(my_force_label_space)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.modal_participation(
    ...     v_real=my_v_real,
    ...     v_imag=my_v_imag,
    ...     mode_shapes=my_mode_shapes,
    ...     ponderation=my_ponderation,
    ...     force_label_space=my_force_label_space,
    ... )

    >>> # Get output data
    >>> result_output = op.outputs.output()
    """

    def __init__(
        self,
        v_real=None,
        v_imag=None,
        mode_shapes=None,
        ponderation=None,
        force_label_space=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="harmonic::modal_participation", config=config, server=server
        )
        self._inputs = InputsModalParticipation(self)
        self._outputs = OutputsModalParticipation(self)
        if v_real is not None:
            self.inputs.v_real.connect(v_real)
        if v_imag is not None:
            self.inputs.v_imag.connect(v_imag)
        if mode_shapes is not None:
            self.inputs.mode_shapes.connect(mode_shapes)
        if ponderation is not None:
            self.inputs.ponderation.connect(ponderation)
        if force_label_space is not None:
            self.inputs.force_label_space.connect(force_label_space)

    @staticmethod
    def _spec():
        description = """Compute the modal participation factor for a given vector field V,
            defined as  sum_i ( V .dot. mode_shape_i * ponderation )."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="v_real",
                    type_names=["field"],
                    optional=False,
                    document="""Real part of field v""",
                ),
                1: PinSpecification(
                    name="v_imag",
                    type_names=["field"],
                    optional=False,
                    document="""Imag part of field v""",
                ),
                2: PinSpecification(
                    name="mode_shapes",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                3: PinSpecification(
                    name="ponderation",
                    type_names=["field"],
                    optional=True,
                    document="""""",
                ),
                4: PinSpecification(
                    name="force_label_space",
                    type_names=["label_space"],
                    optional=True,
                    document="""If set, will force a label space for output
        result.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="output",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="harmonic::modal_participation", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsModalParticipation
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsModalParticipation
        """
        return super().outputs


class InputsModalParticipation(_Inputs):
    """Intermediate class used to connect user inputs to
    modal_participation operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.modal_participation()
    >>> my_v_real = dpf.Field()
    >>> op.inputs.v_real.connect(my_v_real)
    >>> my_v_imag = dpf.Field()
    >>> op.inputs.v_imag.connect(my_v_imag)
    >>> my_mode_shapes = dpf.FieldsContainer()
    >>> op.inputs.mode_shapes.connect(my_mode_shapes)
    >>> my_ponderation = dpf.Field()
    >>> op.inputs.ponderation.connect(my_ponderation)
    >>> my_force_label_space = dict()
    >>> op.inputs.force_label_space.connect(my_force_label_space)
    """

    def __init__(self, op: Operator):
        super().__init__(modal_participation._spec().inputs, op)
        self._v_real = Input(modal_participation._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._v_real)
        self._v_imag = Input(modal_participation._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._v_imag)
        self._mode_shapes = Input(modal_participation._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._mode_shapes)
        self._ponderation = Input(modal_participation._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._ponderation)
        self._force_label_space = Input(
            modal_participation._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._force_label_space)

    @property
    def v_real(self):
        """Allows to connect v_real input to the operator.

        Real part of field v

        Parameters
        ----------
        my_v_real : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_participation()
        >>> op.inputs.v_real.connect(my_v_real)
        >>> # or
        >>> op.inputs.v_real(my_v_real)
        """
        return self._v_real

    @property
    def v_imag(self):
        """Allows to connect v_imag input to the operator.

        Imag part of field v

        Parameters
        ----------
        my_v_imag : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_participation()
        >>> op.inputs.v_imag.connect(my_v_imag)
        >>> # or
        >>> op.inputs.v_imag(my_v_imag)
        """
        return self._v_imag

    @property
    def mode_shapes(self):
        """Allows to connect mode_shapes input to the operator.

        Parameters
        ----------
        my_mode_shapes : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_participation()
        >>> op.inputs.mode_shapes.connect(my_mode_shapes)
        >>> # or
        >>> op.inputs.mode_shapes(my_mode_shapes)
        """
        return self._mode_shapes

    @property
    def ponderation(self):
        """Allows to connect ponderation input to the operator.

        Parameters
        ----------
        my_ponderation : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_participation()
        >>> op.inputs.ponderation.connect(my_ponderation)
        >>> # or
        >>> op.inputs.ponderation(my_ponderation)
        """
        return self._ponderation

    @property
    def force_label_space(self):
        """Allows to connect force_label_space input to the operator.

        If set, will force a label space for output
        result.

        Parameters
        ----------
        my_force_label_space : dict

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_participation()
        >>> op.inputs.force_label_space.connect(my_force_label_space)
        >>> # or
        >>> op.inputs.force_label_space(my_force_label_space)
        """
        return self._force_label_space


class OutputsModalParticipation(_Outputs):
    """Intermediate class used to get outputs from
    modal_participation operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.modal_participation()
    >>> # Connect inputs : op.inputs. ...
    >>> result_output = op.outputs.output()
    """

    def __init__(self, op: Operator):
        super().__init__(modal_participation._spec().outputs, op)
        self._output = Output(modal_participation._spec().output_pin(0), 0, op)
        self._outputs.append(self._output)

    @property
    def output(self):
        """Allows to get output output of the operator

        Returns
        ----------
        my_output : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_participation()
        >>> # Connect inputs : op.inputs. ...
        >>> result_output = op.outputs.output()
        """  # noqa: E501
        return self._output



"""
on_coordinates
==============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class on_coordinates(Operator):
    """Evaluates a result on specified coordinates (interpolates results
    inside elements with shape functions).

    Parameters
    ----------
    fields_container : FieldsContainer
    coordinates : Field or FieldsContainer or MeshedRegion or MeshesContainer
    create_support : bool, optional
        If this pin is set to true, then, a support
        associated to the fields consisting
        of points is created
    mapping_on_scoping : bool, optional
        If this pin is set to true, then the mapping
        between the coordinates and the
        fields is created only on the first
        field scoping
    mesh : MeshedRegion or MeshesContainer, optional
        If the first field in input has no mesh in
        support, then the mesh in this pin is
        expected (default is false), if a
        meshes container with several meshes
        is set, it should be on the same
        label spaces as the coordinates
        fields container


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mapping.on_coordinates()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_coordinates = dpf.Field()
    >>> op.inputs.coordinates.connect(my_coordinates)
    >>> my_create_support = bool()
    >>> op.inputs.create_support.connect(my_create_support)
    >>> my_mapping_on_scoping = bool()
    >>> op.inputs.mapping_on_scoping.connect(my_mapping_on_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mapping.on_coordinates(
    ...     fields_container=my_fields_container,
    ...     coordinates=my_coordinates,
    ...     create_support=my_create_support,
    ...     mapping_on_scoping=my_mapping_on_scoping,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        fields_container=None,
        coordinates=None,
        create_support=None,
        mapping_on_scoping=None,
        mesh=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mapping", config=config, server=server)
        self._inputs = InputsOnCoordinates(self)
        self._outputs = OutputsOnCoordinates(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if coordinates is not None:
            self.inputs.coordinates.connect(coordinates)
        if create_support is not None:
            self.inputs.create_support.connect(create_support)
        if mapping_on_scoping is not None:
            self.inputs.mapping_on_scoping.connect(mapping_on_scoping)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Evaluates a result on specified coordinates (interpolates results
            inside elements with shape functions)."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="coordinates",
                    type_names=[
                        "field",
                        "fields_container",
                        "abstract_meshed_region",
                        "meshes_container",
                    ],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="create_support",
                    type_names=["bool"],
                    optional=True,
                    document="""If this pin is set to true, then, a support
        associated to the fields consisting
        of points is created""",
                ),
                3: PinSpecification(
                    name="mapping_on_scoping",
                    type_names=["bool"],
                    optional=True,
                    document="""If this pin is set to true, then the mapping
        between the coordinates and the
        fields is created only on the first
        field scoping""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region", "meshes_container"],
                    optional=True,
                    document="""If the first field in input has no mesh in
        support, then the mesh in this pin is
        expected (default is false), if a
        meshes container with several meshes
        is set, it should be on the same
        label spaces as the coordinates
        fields container""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mapping", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsOnCoordinates
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsOnCoordinates
        """
        return super().outputs


class InputsOnCoordinates(_Inputs):
    """Intermediate class used to connect user inputs to
    on_coordinates operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.on_coordinates()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_coordinates = dpf.Field()
    >>> op.inputs.coordinates.connect(my_coordinates)
    >>> my_create_support = bool()
    >>> op.inputs.create_support.connect(my_create_support)
    >>> my_mapping_on_scoping = bool()
    >>> op.inputs.mapping_on_scoping.connect(my_mapping_on_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(on_coordinates._spec().inputs, op)
        self._fields_container = Input(on_coordinates._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields_container)
        self._coordinates = Input(on_coordinates._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._coordinates)
        self._create_support = Input(on_coordinates._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._create_support)
        self._mapping_on_scoping = Input(on_coordinates._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._mapping_on_scoping)
        self._mesh = Input(on_coordinates._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator.

        Parameters
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def coordinates(self):
        """Allows to connect coordinates input to the operator.

        Parameters
        ----------
        my_coordinates : Field or FieldsContainer or MeshedRegion or MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.coordinates.connect(my_coordinates)
        >>> # or
        >>> op.inputs.coordinates(my_coordinates)
        """
        return self._coordinates

    @property
    def create_support(self):
        """Allows to connect create_support input to the operator.

        If this pin is set to true, then, a support
        associated to the fields consisting
        of points is created

        Parameters
        ----------
        my_create_support : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.create_support.connect(my_create_support)
        >>> # or
        >>> op.inputs.create_support(my_create_support)
        """
        return self._create_support

    @property
    def mapping_on_scoping(self):
        """Allows to connect mapping_on_scoping input to the operator.

        If this pin is set to true, then the mapping
        between the coordinates and the
        fields is created only on the first
        field scoping

        Parameters
        ----------
        my_mapping_on_scoping : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.mapping_on_scoping.connect(my_mapping_on_scoping)
        >>> # or
        >>> op.inputs.mapping_on_scoping(my_mapping_on_scoping)
        """
        return self._mapping_on_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        If the first field in input has no mesh in
        support, then the mesh in this pin is
        expected (default is false), if a
        meshes container with several meshes
        is set, it should be on the same
        label spaces as the coordinates
        fields container

        Parameters
        ----------
        my_mesh : MeshedRegion or MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsOnCoordinates(_Outputs):
    """Intermediate class used to get outputs from
    on_coordinates operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.on_coordinates()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(on_coordinates._spec().outputs, op)
        self._fields_container = Output(on_coordinates._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.on_coordinates()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container
